/* global jasmine expect Q ZhPreferences*/
describe('ZhPreferences', function(){

    var repositoryFactory;
    var requestEngine;
    var requestFactory;
    var zhPromise;
    var query;
    var zhPreferences;

    beforeEach(function(){
        zhPromise = Q;
        repositoryFactory = jasmine.createSpyObj('repositoryFactory', ['factory']);
        requestEngine     = jasmine.createSpyObj('requestEngine', ['doRequest']);
        requestFactory    = jasmine.createSpyObj('requestFactory', ['factory']);
        ViewMode = {
            VIEW: "view"
        };

        query = jasmine.createSpyObj('query', ['build']);
        zhPreferences = new ZhPreferences(repositoryFactory, requestEngine, requestFactory, zhPromise, query);
    });

    it('registerRepository', function(){
        zhPreferences.registerRepository('repository');

        expect(repositoryFactory.factory).toHaveBeenCalledWith('repository', 'MEMORY');
    });

    it('savePreferences', function(done){
        var filterRepository = jasmine.createSpyObj('filter', ['save']);

        repositoryFactory.factory.and.callFake(function(name){
            return filterRepository;
        });

        var preferenceType = 'filter';
        var preferences = [{'NAME': '1'}, {'NAME': '2'}];

        var expectedRequest = {
            'serviceName': 'zh-preferences#/filter/save',
            'requestType': 'dataSource',
            'dataSource': preferences
        };
        var factoredRequest = {
            'serviceName': 'zh-preferences#/filter/save',
            'requestType': 'dataSource',
            'dataSource': preferences,
            'factoredRequest': true
        };

        var response = {
            dataset: {
                filter: [{'ID': 1}, {'ID': 2}]
            }
        };

        var expectedPreferences = [
            {
                'ID': 1,
                '__is_new': false,
                'changed': false,
                'NAME': '1'
            },
            {
                'ID': 2,
                '__is_new': false,
                'changed': false,
                'NAME': '2'
            }
        ];

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        requestEngine.doRequest.and.callFake(function(){
            return Q.when(response);
        });

        filterRepository.save.and.callFake(function(){
            return expectedPreferences;
        });

        zhPreferences.registerRepository('filter');
        zhPreferences.savePreferences(preferenceType, preferences).then(function(response){
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(filterRepository.save).toHaveBeenCalledWith(expectedPreferences);
            expect(response).toEqual(expectedPreferences);
            done();
        });
    });

    it('savePreferences: request error', function(done){
        var filterRepository = jasmine.createSpyObj('filter', ['save']);

        repositoryFactory.factory.and.callFake(function(name){
            return filterRepository;
        });

        var preferenceType = 'filter';
        var preferences = [{'NAME': '1'}, {'NAME': '2'}];

        var expectedRequest = {
            'serviceName': 'zh-preferences#/filter/save',
            'requestType': 'dataSource',
            'dataSource': preferences
        };
        var factoredRequest = {
            'serviceName': 'zh-preferences#/filter/save',
            'requestType': 'dataSource',
            'dataSource': preferences,
            'factoredRequest': true
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        requestEngine.doRequest.and.callFake(function(){
            return Q.reject({'data': {'error': 'request error'}});
        });

        zhPreferences.registerRepository('filter');
        zhPreferences.savePreferences(preferenceType, preferences).then(null, function(error){
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(filterRepository.save).not.toHaveBeenCalled();
            expect(error).toEqual(new Error('request error'));
            done();
        });
    });

    it('deletePreferences', function(done){
        var filterRepository = jasmine.createSpyObj('filter', ['remove']);

        repositoryFactory.factory.and.callFake(function(name){
            return filterRepository;
        });

        var preferenceType = 'filter';
        var preferences = [{'ID': 1}, {'ID': 2}];

        var expectedRequest = {
            'serviceName': 'zh-preferences#/filter/remove',
            'requestType': 'dataSource',
            'dataSource': preferences
        };

        var factoredRequest = {
            'serviceName': 'zh-preferences#/filter/remove',
            'requestType': 'dataSource',
            'dataSource': preferences,
            'factoredRequest': true
        };

        var response = {
            dataset: {
                filter: [{'ID': 1}, {'ID': 2}]
            }
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        requestEngine.doRequest.and.callFake(function(){
            return Q.when(response);
        });

        var whereMethod = jasmine.createSpy('where');

        query.build.and.callFake(function(){
            return {
                where: whereMethod
            };
        });

        var inMethod = jasmine.createSpy('in');

        whereMethod.and.callFake(function(){
            return {
                in: inMethod
            };
        });

        var expectedRemoveQuery = [
            {
                'columnName': 'ID',
                'operator': 'IN',
                'value': [1, 2]
            }
        ];

        inMethod.and.callFake(function(){
            return expectedRemoveQuery;
        });

        var deleteReturn = [{'ID': 1}, {'ID': 2}];
        filterRepository.remove.and.callFake(function(){
            return deleteReturn;
        });

        zhPreferences.registerRepository('filter');
        zhPreferences.deletePreferences(preferenceType, preferences).then(function(response){
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(filterRepository.remove).toHaveBeenCalledWith(expectedRemoveQuery);
            expect(whereMethod).toHaveBeenCalledWith('ID');
            expect(inMethod).toHaveBeenCalledWith([1, 2]);
            expect(response).toEqual(deleteReturn);
            done();
        });
    });

    it('deletePreferences: request error', function(done){
        var filterRepository = jasmine.createSpyObj('filter', ['remove']);

        repositoryFactory.factory.and.callFake(function(name){
            return filterRepository;
        });

        var preferenceType = 'filter';
        var preferences = [{'ID': 1}, {'ID': 2}];

        var expectedRequest = {
            'serviceName': 'zh-preferences#/filter/remove',
            'requestType': 'dataSource',
            'dataSource': preferences
        };

        var factoredRequest = {
            'serviceName': 'zh-preferences#/filter/remove',
            'requestType': 'dataSource',
            'dataSource': preferences,
            'factoredRequest': true
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        requestEngine.doRequest.and.callFake(function(){
            return Q.reject({'data': {'error': 'request error'}});
        });

        zhPreferences.registerRepository('filter');
        zhPreferences.deletePreferences(preferenceType, preferences).then(null, function(error){
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(filterRepository.remove).not.toHaveBeenCalled();
            expect(error).toEqual(new Error('request error'));
            done();
        });
    });

    it('getPreferences', function(done){
            var preferenceType = 'filter';
            var query = {
                where: function(){
                    return [{'columnName': 'ID', 'value': 1}];
                }
            };

            var filterRepository = jasmine.createSpyObj('filter', ['save', 'remove']);

            repositoryFactory.factory.and.callFake(function(name){
                return filterRepository;
            });

            var expectedRequest = {
                'serviceName': 'zh-preferences#/filter',
                'requestType': 'filterData',
                'filter': [{'columnName': 'ID', 'value': 1}]
            };

            var factoredRequest = {
                'serviceName': 'zh-preferences#/filter',
                'requestType': 'filterData',
                'filter': [{'columnName': 'ID', 'value': 1}],
                'factoredRequest': true
            };

            requestFactory.factory.and.callFake(function(){
                return factoredRequest;
            });

            var filters = [{'ID': 1}, {'ID': 2}, {'ID': 3}];

            var response = {
                'dataset': {'filter': filters}
            };

            requestEngine.doRequest.and.callFake(function(){
                return Q.when(response);
            });

            filterRepository.save.and.callFake(function(){
                return Q.when();
            });

            zhPreferences.registerRepository(preferenceType);
            zhPreferences.getPreferences(preferenceType, query).then(function(response){
                expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
                expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
                expect(filterRepository.remove).toHaveBeenCalledWith(query);
                expect(filterRepository.save).toHaveBeenCalledWith(filters);
                expect(response).toEqual(filters);
                done();
            });
    });

    it('getPreferences: request error', function(done){
        var preferenceType = 'filter';
        var query = {
            where: function(){
                return [{'columnName': 'ID', 'value': 1}];
            }
        };

        var filterRepository = jasmine.createSpyObj('filter', ['save', 'remove']);

        repositoryFactory.factory.and.callFake(function(name){
            return filterRepository;
        });

        var expectedRequest = {
            'serviceName': 'zh-preferences#/filter',
            'requestType': 'filterData',
            'filter': [{'columnName': 'ID', 'value': 1}]
        };

        var factoredRequest = {
            'serviceName': 'zh-preferences#/filter',
            'requestType': 'filterData',
            'filter': [{'columnName': 'ID', 'value': 1}],
            'factoredRequest': true
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        requestEngine.doRequest.and.callFake(function(){
            return Q.reject({'data': {'error': 'request error'}});
        });

        zhPreferences.registerRepository(preferenceType);
        zhPreferences.getPreferences(preferenceType, query).then(null, function(error){
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(filterRepository.remove).not.toHaveBeenCalled();
            expect(filterRepository.save).not.toHaveBeenCalled();
            expect(error).toEqual(new Error('request error'));
            done();
        });
    });

    it('getWidgetPreferences', function(done){
        var preferenceType = 'filter';
        var widgetId = '1234';

        var filters = [{'ID': 1}, {'ID': 2}, {'ID': 3}];

        spyOn(zhPreferences, 'getPreferences').and.callFake(function(){
            return Q.resolve(filters);
        });

        var expectedQuery = 'query';

        var equalsMethod = jasmine.createSpy('equals').and.callFake(function(){
            return expectedQuery;
        });

        var whereMethod = jasmine.createSpy('where').and.callFake(function(){
            return {
                equals: equalsMethod
            };
        });

        query.build.and.callFake(function(){
            return {
                where: whereMethod
            };
        });

        zhPreferences.getWidgetPreferences(preferenceType, widgetId).then(function(response){
            expect(zhPreferences.getPreferences).toHaveBeenCalledWith(preferenceType, expectedQuery);
            expect(response).toEqual(filters);
            done();
        });
    });

    it('getUserID before setUserID', function(done){
        var userID = '123';
        zhPreferences.getUserID().then(function(result){
            expect(result).toEqual(userID);
            done();
        });

        zhPreferences.setUserID(userID);
    });

    it('getUserID after setUserID', function(done){
        var userID = '123';
        zhPreferences.setUserID(userID);

        zhPreferences.getUserID().then(function(result){
            expect(result).toEqual(userID);
            done();
        });
    });

    it('unsetUser', function(){
        var filterRepository = jasmine.createSpyObj('filter', ['clearAll']);
        var layoutRepository = jasmine.createSpyObj('layout', ['clearAll']);

        var repositories = {
            'filter': filterRepository,
            'layout': layoutRepository
        };

        repositoryFactory.factory.and.callFake(function(name){
            return repositories[name];
        });

        zhPreferences.registerRepository('filter');
        zhPreferences.registerRepository('layout');

        zhPreferences.unsetUser();

        expect(filterRepository.clearAll).toHaveBeenCalled();
        expect(layoutRepository.clearAll).toHaveBeenCalled();
    });

    it('setUserID: calling multiple times', function(done){
        var userID = '123';
        zhPreferences.setUserID(userID);

        zhPreferences.getUserID().then(function(result){
            expect(result).toEqual(userID);
            zhPreferences.getUserID().then(function(secondResult){
                expect(secondResult).toEqual(userID);
                done();
            });
            zhPreferences.setUserID('another user');
        });
    });

    it('setUserID: calling multiple times after unsetting user', function(done){
        var userID = '123';
        zhPreferences.setUserID(userID);

        zhPreferences.getUserID().then(function(result){
            expect(result).toEqual(userID);
            zhPreferences.unsetUser();
            zhPreferences.getUserID().then(function(secondResult){
                expect(secondResult).toEqual('another user');
                done();
            });
            zhPreferences.setUserID('another user');
        });
    });

    it('getDefaultWidgetPreferences: with value being returned on request', function(done){
        var widgetID = '123';
        var queryConditions = [{COLUMN: 'value'}];
        var expectedQuery = jasmine.createSpyObj('query', ['where']);

        expectedQuery.where.and.callFake(function(){
            return queryConditions;
        });

        var equalsMethod = jasmine.createSpy('equals').and.callFake(function(){
            return expectedQuery;
        });

        var whereMethod = jasmine.createSpy('where').and.callFake(function(){
            return {
                equals: equalsMethod
            };
        });

        query.build.and.callFake(function(){
            return {
                where: whereMethod
            };
        });

        var expectedRequest = {
            'serviceName': 'zh-preferences#/default-preferences',
            'requestType': 'filterData',
            'filter': queryConditions
        };

        var factoredRequest = {
            'serviceName': 'zh-preferences#/default-preferences',
            'requestType': 'filterData',
            'filter': queryConditions,
            'factoredRequest': true
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        var defaultPreferences = {'ID': 1};

        var response = {
            dataset: {
                user_widget: [defaultPreferences]
            }
        };

        requestEngine.doRequest.and.callFake(function(){
            return Q.resolve(response);
        });

        zhPreferences.getDefaultWidgetPreferences(widgetID).then(function(result){
            expect(whereMethod).toHaveBeenCalledWith('WIDGET_ID');
            expect(equalsMethod).toHaveBeenCalledWith(widgetID);
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(result).toEqual(defaultPreferences);
            done();
        });
    });

    it('getDefaultWidgetPreferences: with value not being returned on request', function(done){
        var widgetID = '123';

        var queryConditions = [{COLUMN: 'value'}];

        var expectedQuery = jasmine.createSpyObj('query', ['where']);

        expectedQuery.where.and.callFake(function(){
            return queryConditions;
        });

        var equalsMethod = jasmine.createSpy('equals').and.callFake(function(){
            return expectedQuery;
        });

        var whereMethod = jasmine.createSpy('where').and.callFake(function(){
            return {
                equals: equalsMethod
            };
        });

        query.build.and.callFake(function(){
            return {
                where: whereMethod
            };
        });

        var expectedRequest = {
            'serviceName': 'zh-preferences#/default-preferences',
            'requestType': 'filterData',
            'filter': queryConditions
        };

        var factoredRequest = {
            'serviceName': 'zh-preferences#/default-preferences',
            'requestType': 'filterData',
            'filter': queryConditions,
            'factoredRequest': true
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        var defaultPreferences = {
            '__is_new': true,
            'ID': null,
            'WIDGET_ID': widgetID,
            'DEFAULT_FILTER': null,
            'DEFAULT_LAYOUT': null,
            'DEFAULT_VIEW': null,
            'LAST_MODIFIED_DATE': null
        };

        var response = {
            dataset: {
                user_widget: []
            }
        };

        requestEngine.doRequest.and.callFake(function(){
            return Q.resolve(response);
        });

        zhPreferences.getDefaultWidgetPreferences(widgetID).then(function(result){
            expect(whereMethod).toHaveBeenCalledWith('WIDGET_ID');
            expect(equalsMethod).toHaveBeenCalledWith(widgetID);
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(result).toEqual(defaultPreferences);
            done();
        });
    });

    it('getDefaultWidgetPreferences: calling multiple times for same widget', function(done){
        var widgetID = '123';

        var queryConditions = [{COLUMN: 'value'}];

        var expectedQuery = jasmine.createSpyObj('query', ['where']);

        expectedQuery.where.and.callFake(function(){
            return queryConditions;
        });

        var equalsMethod = jasmine.createSpy('equals').and.callFake(function(){
            return expectedQuery;
        });

        var whereMethod = jasmine.createSpy('where').and.callFake(function(){
            return {
                equals: equalsMethod
            };
        });

        query.build.and.callFake(function(){
            return {
                where: whereMethod
            };
        });

        var expectedRequest = {
            'serviceName': 'zh-preferences#/default-preferences',
            'requestType': 'filterData',
            'filter': queryConditions
        };

        var factoredRequest = {
            'serviceName': 'zh-preferences#/default-preferences',
            'requestType': 'filterData',
            'filter': queryConditions,
            'factoredRequest': true
        };

        requestFactory.factory.and.callFake(function(){
            return factoredRequest;
        });

        var defaultPreferences = {
            '__is_new': true,
            'ID': null,
            'WIDGET_ID': widgetID,
            'DEFAULT_FILTER': null,
            'DEFAULT_LAYOUT': null,
            'DEFAULT_VIEW': null,
            'LAST_MODIFIED_DATE': null
        };

        var response = {
            dataset: {
                user_widget: []
            }
        };

        requestEngine.doRequest.and.callFake(function(){
            return Q.resolve(response);
        });

        zhPreferences.getDefaultWidgetPreferences(widgetID).then(function(result){
            expect(whereMethod).toHaveBeenCalledWith('WIDGET_ID');
            expect(equalsMethod).toHaveBeenCalledWith(widgetID);
            expect(requestFactory.factory).toHaveBeenCalledWith(expectedRequest);
            expect(requestEngine.doRequest).toHaveBeenCalledWith(factoredRequest);
            expect(result).toEqual(defaultPreferences);
        }).then(function(){
            return zhPreferences.getDefaultWidgetPreferences(widgetID);
        }).then(function(result){
            expect(whereMethod.calls.count()).toEqual(1);
            expect(equalsMethod.calls.count()).toEqual(1);
            expect(requestFactory.factory.calls.count()).toEqual(1);
            expect(requestEngine.doRequest.calls.count()).toEqual(1);
            expect(result).toEqual(defaultPreferences);
            done();
        });
    });

    it('validateWidget: return true when widget is valid to use preferences', function(){
        var widget = {
            id: 'id',
            template: 'widgetTemplate',
            container: {
               id: 'containerId'
            }
        };
        expect(zhPreferences.validateWidget(widget)).toBeTruthy();
    });

    it('validateWidget: return false when widget is master-detail in view mode', function(){
        var widget = {
            id: 'id',
            mode: 'view',
            template: 'master_detail/grid.html',
            container: {
                id: 'containerId'
            }
        };
        expect(zhPreferences.validateWidget(widget)).toBeFalsy();

        widget.template = 'master_detail/list.html';
        expect(zhPreferences.validateWidget(widget)).toBeFalsy();

        widget.template = 'master_detail/tree/grid.html';
        expect(zhPreferences.validateWidget(widget)).toBeFalsy();
    });

    it('validateWidget: should throw error if widget does not have an ID', function(){
        var widget = {
            id: null,
            name: 'test_widget',
            template: 'widgetTemplate',
            container: {
                id: 'containerId'
            }
        };
        var expectedError = new Error('Widget\'s must have "id" to use Preferences. Widget with name "test_widget" does not have an id.');
        expect(function(){
            zhPreferences.validateWidget(widget);
        }).toThrow(expectedError);
    });

    it('validateWidget: should throw error if widget.container does not have an ID', function(){
        var widget = {
            id: 'id',
            template: 'widgetTemplate',
            container: {
                id: null
            }
        };
        var expectedError = new Error('Container\'s must have "id" to use Preferences.');
        expect(function(){
            zhPreferences.validateWidget(widget);
        }).toThrow(expectedError);
    });

    describe('changeDefaultPreferences()', function(){
        var property;
        var configWidget;
        var preference;
        var defaultPreferences;

        beforeEach(function(){
            Util = {
                getNestedProperty: function(obj, pathToProperty) {
                    var none = {};
                    var got = pathToProperty.reduce(function(prev, next) {
                        return prev[next] ? prev[next] : none;
                    }, obj);
                    return got === none ? undefined : got;
                }
            };
            property = 'PROPERTY_NAME';
            configWidget = {
                id: 'configWidgetId',
                dataSource: {
                    data: [
                        {ID: 1},
                        {ID: 'preferenceId'},
                        {ID: 3}
                    ]
                },
                listWidget: {
                    id: 'widgetId'
                }
            };
            preference = {
                ID: 'preferenceId',
                PROPERTY_NAME: true
            };
            defaultPreferences = {};

            spyOn(zhPreferences, 'getDefaultWidgetPreferences').and.callFake(function(){
                return Q.when(defaultPreferences);
            });

            var factoredRequest = {
                'serviceName': 'zh-preferences#/default-preferences/save',
                'requestType': 'dataSource',
                'dataSource': [defaultPreferences],
                'factoredRequest': true
            };

            requestFactory.factory.and.callFake(function(){
                return factoredRequest;
            });

            var response = {
                dataset: {
                    user_widget: [{ID: 'savedPreferenceId'}]
                }
            };

            requestEngine.doRequest.and.callFake(function(){
                return response;
            });
        });

        it('set default preference', function(done){
            var expectedRow = {
                __is_new: false,
                PROPERTY_NAME: false
            };
            var expectedSavedRow = {
                __is_new: false,
                PROPERTY_NAME: true
            };

            zhPreferences.changeDefaultPreferences(configWidget, property, preference).then(function(result){
                expect(zhPreferences.getDefaultWidgetPreferences).toHaveBeenCalledWith(configWidget.listWidget.id);
                expect(requestFactory.factory).toHaveBeenCalled();
                expect(requestEngine.doRequest).toHaveBeenCalled();

                expect(result.ID).toBe('savedPreferenceId');
                expect(result.PROPERTY_NAME).toBe('preferenceId');

                expect(configWidget.dataSource.data[0]).toEqual(jasmine.objectContaining(expectedRow));
                expect(configWidget.dataSource.data[1]).toEqual(jasmine.objectContaining(expectedSavedRow));
                expect(configWidget.dataSource.data[2]).toEqual(jasmine.objectContaining(expectedRow));
                done();
            });
        });

        it('unset default preference', function(done){
            var expectedRow = {
                __is_new: false,
                PROPERTY_NAME: false
            };

            preference.PROPERTY_NAME = false;

            zhPreferences.changeDefaultPreferences(configWidget, property, preference).then(function(result){
                expect(zhPreferences.getDefaultWidgetPreferences).toHaveBeenCalledWith(configWidget.listWidget.id);
                expect(requestFactory.factory).toHaveBeenCalled();
                expect(requestEngine.doRequest).toHaveBeenCalled();

                expect(result.ID).toBe('savedPreferenceId');
                expect(result.PROPERTY_NAME).toBe(null);

                expect(configWidget.dataSource.data[0]).toEqual(jasmine.objectContaining(expectedRow));
                expect(configWidget.dataSource.data[1]).toEqual(jasmine.objectContaining(expectedRow));
                expect(configWidget.dataSource.data[2]).toEqual(jasmine.objectContaining(expectedRow));
                done();
            });
        });

        it('error on save', function(done){
            requestEngine.doRequest.and.callFake(function(){
                return Q.reject('error on save');
            });
            zhPreferences.changeDefaultPreferences(configWidget, property, preference).catch(function(result){
                expect(result).toBe('error on save');
                done();
            });
        });

        it('error on getDefaultWidgetPreferences', function(done){
            zhPreferences.getDefaultWidgetPreferences.and.callFake(function(){
                return Q.reject('error on getDefaultWidgetPreferences');
            });

            zhPreferences.changeDefaultPreferences(configWidget, property, preference).catch(function(result){
                expect(result).toBe('error on getDefaultWidgetPreferences');
                done();
            });
        });
    });

    it('downloadPreferenceFile', function(done){
        var preference = {
            'ID': 1
        };
        var fileName = 'preference.json';

        var setAttributeCalled = 0;
        var clickCalled = 0;
        var removeCalled = 0;
        oldCreateElement = document.createElement;
        document.createElement = jasmine.createSpy('createElement').and.callFake(function(el){
            return {
                setAttribute: function(){ setAttributeCalled++; },
                click: function(){ clickCalled++; },
                remove: function(){ removeCalled++; }
            };
        });

        zhPreferences.downloadPreferenceFile(preference, fileName).then(function(){
            expect(document.createElement).toHaveBeenCalled();
            expect(setAttributeCalled).toBe(2);
            expect(clickCalled).toBe(1);
            expect(removeCalled).toBe(1);
            document.createElement = oldCreateElement;
            done();
        });
    });
});